@@ -1,11 +1,11 @@
 from ocsmesh.ops.combine_geom import GeomCombine
 from ocsmesh.ops.combine_hfun import HfunCombine
 
-combine_geometry = lambda *args, **kwargs: GeomCombine(
-        *args, **kwargs).run()
+def combine_geometry(*args, **kwargs):
+    return GeomCombine(*args, **kwargs).run()
 
-combine_hfun = lambda *args, **kwargs: HfunCombine(
-        *args, **kwargs).run()
+def combine_hfun(*args, **kwargs):
+    return HfunCombine(*args, **kwargs).run()
 
 __all__ = [
         "combine_geometry",
